@@ -39,7 +39,7 @@
 #include "applet.h"
 #include "applet-device-cdma.h"
 #include "utils.h"
-#include "mobile-wizard.h"
+#include "nm-mobile-wizard.h"
 #include "applet-dialogs.h"
 #include "nma-marshal.h"
 #include "nmn-mobile-providers.h"
@@ -94,9 +94,9 @@ typedef struct {
 } AutoCdmaWizardInfo;
 
 static void
-mobile_wizard_done (MobileWizard *wizard,
+mobile_wizard_done (NMAMobileWizard *wizard,
                     gboolean canceled,
-                    MobileWizardAccessMethod *method,
+                    NMAMobileWizardAccessMethod *method,
                     gpointer user_data)
 {
 	AutoCdmaWizardInfo *info = user_data;
@@ -155,7 +155,7 @@ done:
 	(*(info->callback)) (connection, TRUE, canceled, info->callback_data);
 
 	if (wizard)
-		mobile_wizard_destroy (wizard);
+		nma_mobile_wizard_destroy (wizard);
 	g_free (info);
 }
 
@@ -163,23 +163,23 @@ static gboolean
 do_mobile_wizard (AppletNewAutoConnectionCallback callback,
                   gpointer callback_data)
 {
-	MobileWizard *wizard;
+	NMAMobileWizard *wizard;
 	AutoCdmaWizardInfo *info;
-	MobileWizardAccessMethod *method;
+	NMAMobileWizardAccessMethod *method;
 
 	info = g_malloc0 (sizeof (AutoCdmaWizardInfo));
 	info->callback = callback;
 	info->callback_data = callback_data;
 
-	wizard = mobile_wizard_new (NULL, NULL, NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO, FALSE,
-	                            mobile_wizard_done, info);
+	wizard = nma_mobile_wizard_new (NULL, NULL, NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO, FALSE,
+									mobile_wizard_done, info);
 	if (wizard) {
-		mobile_wizard_present (wizard);
+		nma_mobile_wizard_present (wizard);
 		return TRUE;
 	}
 
 	/* Fall back to something */
-	method = g_malloc0 (sizeof (MobileWizardAccessMethod));
+	method = g_malloc0 (sizeof (NMAMobileWizardAccessMethod));
 	method->devtype = NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO;
 	method->provider_name = _("CDMA");
 	mobile_wizard_done (NULL, FALSE, method, info);
