@@ -19,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -111,9 +111,9 @@ ui_to_setting (NMConnectionEditor *editor)
 	NMSettingConnection *s_con;
 	GtkWidget *widget;
 	const char *name;
-	gboolean autoconnect = FALSE;
+	gboolean autoconnect = FALSE, everyone = FALSE;
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (editor->connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (editor->connection);
 	g_assert (s_con);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (editor->builder, "connection_name"));
@@ -129,18 +129,12 @@ ui_to_setting (NMConnectionEditor *editor)
 	autoconnect = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 	g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_AUTOCONNECT, autoconnect, NULL);
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->all_checkbutton))) {
-		/* Visible to everyone */
-		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_PERMISSIONS, NULL, NULL);
-	} else {
-		GSList *users = NULL;
-		char buf[75] = "user:";
-
-		/* Visible only to this user */
-		strcat (buf, g_get_user_name ());
-		users = g_slist_append (users, buf);
-		g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_PERMISSIONS, users, NULL);
-		g_slist_free (users);
+	/* Handle visibility */
+	everyone = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->all_checkbutton));
+	g_object_set (G_OBJECT (s_con), NM_SETTING_CONNECTION_PERMISSIONS, NULL, NULL);
+	if (everyone == FALSE) {
+		/* Only visible to this user */
+		nm_setting_connection_add_permission (s_con, "user", g_get_user_name (), NULL);
 	}
 
 	return TRUE;
@@ -271,18 +265,20 @@ all_checkbutton_toggled_cb (GtkWidget *widget, NMConnectionEditor *editor)
 	connection_editor_validate (editor);
 }
 
-#define NMCE_UI UIDIR "/nm-connection-editor.ui"
-
 static void
 nm_connection_editor_init (NMConnectionEditor *editor)
 {
 	GtkWidget *dialog;
 	GError *error = NULL;
+	const char *objects[] = { "nm-connection-editor", NULL };
 
 	editor->builder = gtk_builder_new ();
 
-	if (!gtk_builder_add_from_file (editor->builder, NMCE_UI, &error)) {
-		g_warning ("Couldn't load builder file " NMCE_UI ": %s", error->message);
+	if (!gtk_builder_add_objects_from_file (editor->builder,
+	                                        UIDIR "/nm-connection-editor.ui",
+	                                        (char **) objects,
+	                                        &error)) {
+		g_warning ("Couldn't load builder file " UIDIR "/nm-connection-editor.ui: %s", error->message);
 		g_error_free (error);
 
 		dialog = gtk_message_dialog_new (NULL, 0,
@@ -437,16 +433,58 @@ nm_connection_editor_get_connection (NMConnectionEditor *editor)
 	return editor->orig_connection;
 }
 
+static void
+update_secret_flags (NMSetting *setting,
+                     const char *key,
+                     const GValue *value,
+                     GParamFlags flags,
+                     gpointer user_data)
+{
+	gboolean everyone = !!GPOINTER_TO_UINT (user_data);
+	NMSettingSecretFlags secret_flags = NM_SETTING_SECRET_FLAG_NONE;
+
+	if (!(flags & NM_SETTING_PARAM_SECRET))
+		return;
+
+	/* VPN connections never get changed */
+	if (NM_IS_SETTING_VPN (setting))
+		return;
+
+	/* 802.1x passwords don't get changed either */
+	if (NM_IS_SETTING_802_1X (setting)) {
+		if (   g_strcmp0 (key, NM_SETTING_802_1X_PASSWORD) == 0
+		    || g_strcmp0 (key, NM_SETTING_802_1X_PRIVATE_KEY_PASSWORD) == 0
+		    || g_strcmp0 (key, NM_SETTING_802_1X_PHASE2_PRIVATE_KEY_PASSWORD) == 0)
+			return;
+	}
+
+	nm_setting_get_secret_flags (setting, key, &secret_flags, NULL);
+	if (everyone)
+		secret_flags &= ~NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+	else
+		secret_flags |= NM_SETTING_SECRET_FLAG_AGENT_OWNED;
+
+	nm_setting_set_secret_flags (setting, key, secret_flags, NULL);
+}
+
 gboolean
 nm_connection_editor_update_connection (NMConnectionEditor *editor, GError **error)
 {
 	GHashTable *settings;
+	gboolean everyone = FALSE;
 
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
 
 	if (!nm_connection_verify (editor->connection, error))
 		return FALSE;
 
+	/* Update secret flags at the end after all other settings have updated,
+	 * otherwise the secret flags we set here might be overwritten during
+	 * setting validation.
+	 */
+	everyone = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->all_checkbutton));
+	nm_connection_for_each_setting_value (editor->connection, update_secret_flags, GUINT_TO_POINTER (everyone));
+
 	/* Copy the modified connection to the original connection */
 	settings = nm_connection_to_hash (editor->connection, NM_SETTING_HASH_FLAG_ALL);
 	nm_connection_replace_settings (editor->orig_connection, settings, NULL);
@@ -606,25 +644,6 @@ get_secrets_for_page (NMConnectionEditor *self,
 {
 	GetSecretsInfo *info;
 
-	if (!setting_name) {
-		/* page doesn't need any secrets */
-		ce_page_complete_init (page, NULL, NULL, NULL);
-		return;
-	}
-
-	/* Try to get secrets from ->orig_connection, because it's the one that
-	 * implements NMSettingsConnectionInterface and can respond to requests for
-	 * its secrets.  ->connection is a plain NMConnection copy of ->orig_connection
-	 * which is the connection that's actually changed when the user clicks stuff.
-	 * When creating importing or creating new connections though, ->orig_connection
-	 * is an NMConnection because it hasn't been exported over D-Bus yet, so we
-	 * can't ask it for secrets, because it doesn't implement NMSettingsConnectionInterface.
-	 */
-	if (!NM_IS_REMOTE_CONNECTION (self->orig_connection)) {
-		ce_page_complete_init (page, setting_name, NULL, NULL);
-		return;
-	}
-
 	info = g_malloc0 (sizeof (GetSecretsInfo));
 	info->self = self;
 	info->page = page;
@@ -652,6 +671,8 @@ get_secrets_for_page (NMConnectionEditor *self,
 	}
 }
 
+#define SECRETS_TAG "secrets-setting-name"
+
 static gboolean
 add_page (NMConnectionEditor *editor,
           CEPageNewFunc func,
@@ -665,20 +686,18 @@ add_page (NMConnectionEditor *editor,
 	g_return_val_if_fail (func != NULL, FALSE);
 	g_return_val_if_fail (connection != NULL, FALSE);
 
-	page = (*func) (connection, GTK_WINDOW (editor->window), &secrets_setting_name, error);
-	if (!page)
-		return FALSE;
+	page = (*func) (connection, GTK_WINDOW (editor->window), editor->client, &secrets_setting_name, error);
+	if (page) {
+		g_object_set_data_full (G_OBJECT (page),
+		                        SECRETS_TAG,
+		                        g_strdup (secrets_setting_name),
+		                        g_free);
 
-	editor->initializing_pages = g_slist_prepend (editor->initializing_pages, page);
-	g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
-	g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
-
-	/* Request any secrets the page might require; or if it doesn't want any,
-	 * let the page initialize.
-	 */
-	get_secrets_for_page (editor, page, secrets_setting_name);
-
-	return TRUE;
+		editor->initializing_pages = g_slist_append (editor->initializing_pages, page);
+		g_signal_connect (page, "changed", G_CALLBACK (page_changed), editor);
+		g_signal_connect (page, "initialized", G_CALLBACK (page_initialized), editor);
+	}
+	return !!page;
 }
 
 static gboolean
@@ -689,6 +708,7 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 	NMSettingConnection *s_con;
 	const char *connection_type;
 	gboolean success = FALSE;
+	GSList *iter, *copy;
 
 	g_return_val_if_fail (NM_IS_CONNECTION_EDITOR (editor), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (orig_connection), FALSE);
@@ -750,6 +770,38 @@ nm_connection_editor_set_connection (NMConnectionEditor *editor,
 		g_warning ("Unhandled setting type '%s'", connection_type);
 	}
 
+	/* After all pages are created, then kick off secrets requests that any
+	 * the pages may need to make; if they don't need any secrets, then let
+	 * them finish initialization.  The list might get modified during the loop
+	 * which is why copy the list here.
+	 */
+	copy = g_slist_copy (editor->initializing_pages);
+	for (iter = copy; iter; iter = g_slist_next (iter)) {
+		CEPage *page = CE_PAGE (iter->data);
+		const char *setting_name = g_object_get_data (G_OBJECT (page), SECRETS_TAG);
+
+		if (!setting_name) {
+			/* page doesn't need any secrets */
+			ce_page_complete_init (page, NULL, NULL, NULL);
+		} else if (!NM_IS_REMOTE_CONNECTION (editor->orig_connection)) {
+			/* We want to get secrets using ->orig_connection, since that's the
+			 * remote connection which can actually respond to secrets requests.
+			 * ->connection is a plain NMConnection copy of ->orig_connection
+			 * which is what gets changed when users modify anything.  But when
+			 * creating or importing, ->orig_connection will be an NMConnection
+			 * since the new connection hasn't been added to NetworkManager yet.
+			 * So basically, skip requesting secrets if the connection can't
+			 * handle a secrets request.
+			 */
+			ce_page_complete_init (page, setting_name, NULL, NULL);
+		} else {
+			/* Page wants secrets, get them */
+			get_secrets_for_page (editor, page, setting_name);
+		}
+		g_object_set_data (G_OBJECT (page), SECRETS_TAG, NULL);
+	}
+	g_slist_free (copy);
+
 	/* set the UI */
 	recheck_initialization (editor);
 	success = TRUE;
