@@ -205,7 +205,8 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
                        NMConnection *connection,
                        EAPMethodSimpleType type,
                        gboolean phase2,
-                       gboolean is_editor)
+                       gboolean is_editor,
+                       gboolean secrets_only)
 {
 	EAPMethod *parent;
 	EAPMethodSimple *method;
@@ -241,6 +242,9 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 			gtk_entry_set_text (GTK_ENTRY (widget), nm_setting_802_1x_get_identity (s_8021x));
 	}
 
+	if (secrets_only)
+		gtk_widget_set_sensitive (widget, FALSE);
+
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_simple_password_entry"));
 	g_assert (widget);
 	g_signal_connect (G_OBJECT (widget), "changed",
@@ -264,6 +268,9 @@ eap_method_simple_new (WirelessSecurity *ws_parent,
 		                  method);
 	}
 
+	if (secrets_only)
+		gtk_widget_hide (widget);
+
 	if (s_8021x) {
 		NMSettingSecretFlags flags = NM_SETTING_SECRET_FLAG_NONE;
 
