@@ -196,7 +196,8 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 static GtkWidget *
 inner_auth_combo_init (EAPMethodTTLS *method,
                        NMConnection *connection,
-                       NMSetting8021x *s_8021x)
+                       NMSetting8021x *s_8021x,
+                       gboolean secrets_only)
 {
 	EAPMethod *parent = (EAPMethod *) method;
 	GtkWidget *combo;
@@ -222,7 +223,8 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	                                connection,
 	                                EAP_METHOD_SIMPLE_TYPE_PAP,
 	                                TRUE,
-	                                method->is_editor);
+	                                method->is_editor,
+	                                secrets_only);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("PAP"),
@@ -238,7 +240,8 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	                                   connection,
 	                                   EAP_METHOD_SIMPLE_TYPE_MSCHAP,
 	                                   TRUE,
-	                                   method->is_editor);
+	                                   method->is_editor,
+	                                   secrets_only);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("MSCHAP"),
@@ -254,7 +257,7 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	                                      connection,
 	                                      EAP_METHOD_SIMPLE_TYPE_MSCHAP_V2,
 	                                      TRUE,
-	                                      method->is_editor);
+	                                      method->is_editor, secrets_only);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("MSCHAPv2"),
@@ -270,7 +273,8 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	                                 connection,
 	                                 EAP_METHOD_SIMPLE_TYPE_CHAP,
 	                                 TRUE,
-	                                 method->is_editor);
+	                                 method->is_editor,
+	                                 secrets_only);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    I_NAME_COLUMN, _("CHAP"),
@@ -307,7 +311,8 @@ update_secrets (EAPMethod *parent, NMConnection *connection)
 EAPMethodTTLS *
 eap_method_ttls_new (WirelessSecurity *ws_parent,
                      NMConnection *connection,
-                     gboolean is_editor)
+                     gboolean is_editor,
+                     gboolean secrets_only)
 {
 	EAPMethod *parent;
 	EAPMethodTTLS *method;
@@ -363,9 +368,24 @@ eap_method_ttls_new (WirelessSecurity *ws_parent,
 	                  (GCallback) wireless_security_changed_cb,
 	                  ws_parent);
 
-	widget = inner_auth_combo_init (method, connection, s_8021x);
+	widget = inner_auth_combo_init (method, connection, s_8021x, secrets_only);
 	inner_auth_combo_changed_cb (widget, (gpointer) method);
 
+	if (secrets_only) {
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_anon_identity_label"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_anon_identity_entry"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_ca_cert_label"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_ca_cert_button"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_label"));
+		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
+		gtk_widget_hide (widget);
+	}
+
 	return method;
 }
 
