@@ -18,9 +18,10 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  */
 
+#include "config.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -31,11 +32,12 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
+
 #include <nm-setting-connection.h>
 #include <nm-setting-8021x.h>
 #include "eap-method.h"
-#include "gconf-helpers.h"
-
 
 GType
 eap_method_get_g_type (void)
@@ -103,6 +105,43 @@ typedef struct {
 	NMConnection *connection;
 } NagDialogResponseInfo;
 
+static void
+nag_dialog_destroyed (gpointer data, GObject *dialog_ptr)
+{
+	NagDialogResponseInfo *info = (NagDialogResponseInfo *) data;
+
+	memset (info, '\0', sizeof (NagDialogResponseInfo));
+	g_free (info);
+}
+
+static char *
+_get_ca_ignore_path (const char *uuid, gboolean phase2)
+{
+	return g_strdup_printf ("/apps/nm-applet/%s/%s",
+	                        phase2 ? "ignore-phase2-ca-cert" : "ignore-ca-cert",
+	                        uuid);
+}
+
+static void
+_set_ignore_ca_cert (const char *uuid, gboolean phase2, gboolean ignore)
+{
+	GConfClient *client;
+	char *key = NULL;
+
+	g_return_if_fail (uuid != NULL);
+
+	client = gconf_client_get_default ();
+
+	key = _get_ca_ignore_path (uuid, phase2);
+	if (ignore)
+		gconf_client_set_bool (client, key, ignore, NULL);
+	else
+		gconf_client_unset (client, key, NULL);
+	g_free (key);
+
+	g_object_unref (client);
+}
+
 static void
 nag_dialog_response_cb (GtkDialog *nag_dialog,
                         gint response,
@@ -115,19 +154,29 @@ nag_dialog_response_cb (GtkDialog *nag_dialog,
 
 	if (response == GTK_RESPONSE_NO) {
 		/* Grab the value of the "don't bother me" checkbox */
-		widget = glade_xml_get_widget (method->nag_dialog_xml, "ignore_checkbox");
+		widget = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "ignore_checkbox"));
 		g_assert (widget);
 
 		method->ignore_ca_cert = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
-		/* Set the value to connection. It will be stored when connection is written (in nm_gconf_write_connection()) */
-		g_object_set_data (G_OBJECT (connection), IGNORE_CA_CERT_TAG, GUINT_TO_POINTER (method->ignore_ca_cert));
+		/* And save it */
+		_set_ignore_ca_cert (nm_connection_get_uuid (connection),
+		                     method->phase2,
+		                     method->ignore_ca_cert);
 	}
 
 	gtk_widget_hide (GTK_WIDGET (nag_dialog));
-	g_free (info);
 }
 
+static gboolean 
+nag_dialog_delete_event_cb (GtkDialog *nag_dialog, GdkEvent *e, gpointer user_data) 
+{ 
+	// FIXME?: By emitting response signal, dismissing nag dialog with upper right "x" icon,
+	// Alt-F4, or Esc would have the same behaviour as clicking "Ignore" button.
+	//g_signal_emit_by_name (nag_dialog, "response", GTK_RESPONSE_NO, user_data);
+	return TRUE;  /* do not destroy */
+} 
+
 GtkWidget *
 eap_method_nag_user (EAPMethod *method)
 {
@@ -140,12 +189,12 @@ eap_method_nag_user (EAPMethod *method)
 		return NULL;
 
 	/* Checkbox should be unchecked each time dialog comes up */
-	widget = glade_xml_get_widget (method->nag_dialog_xml, "ignore_checkbox");
+	widget = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "ignore_checkbox"));
 	g_assert (widget);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 
 	/* Nag the user if the CA Cert is blank, since it's a security risk. */
-	widget = glade_xml_get_widget (method->xml, method->ca_cert_chooser);
+	widget = GTK_WIDGET (gtk_builder_get_object (method->builder, method->ca_cert_chooser));
 	g_assert (widget);
 	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
 	if (filename != NULL) {
@@ -157,24 +206,45 @@ eap_method_nag_user (EAPMethod *method)
 	return method->nag_dialog;
 }
 
+#define NAG_DIALOG_UI UIDIR "/nag-user-dialog.ui"
+
+static gboolean
+_get_ignore_ca_cert (const char *uuid, gboolean phase2)
+{
+	GConfClient *client;
+	char *key = NULL;
+	gboolean ignore = FALSE;
+
+	g_return_val_if_fail (uuid != NULL, FALSE);
+
+	client = gconf_client_get_default ();
+
+	key = _get_ca_ignore_path (uuid, phase2);
+	ignore = gconf_client_get_bool (client, key, NULL);
+	g_free (key);
+
+	g_object_unref (client);
+	return ignore;
+}
+
 gboolean
 eap_method_nag_init (EAPMethod *method,
-                     const char *glade_file,
                      const char *ca_cert_chooser,
-                     NMConnection *connection,
-                     gboolean phase2)
+                     NMConnection *connection)
 {
 	GtkWidget *dialog, *widget;
-	char *text;
 	NagDialogResponseInfo *info;
+	GError *error = NULL;
+	char *text;
 
 	g_return_val_if_fail (method != NULL, FALSE);
-	g_return_val_if_fail (glade_file != NULL, FALSE);
 	g_return_val_if_fail (ca_cert_chooser != NULL, FALSE);
 
-	method->nag_dialog_xml = glade_xml_new (glade_file, "nag_user_dialog", NULL);
-	if (method->nag_dialog_xml == NULL) {
-		g_warning ("Couldn't get nag_user_dialog from glade xml");
+	method->nag_builder = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (method->nag_builder, NAG_DIALOG_UI, &error)) {
+		g_warning ("Couldn't load UI builder file " NAG_DIALOG_UI ": %s",
+		           error->message);
+		g_error_free (error);
 		return FALSE;
 	}
 
@@ -188,18 +258,21 @@ eap_method_nag_init (EAPMethod *method,
 		uuid = nm_setting_connection_get_uuid (s_con);
 		g_assert (uuid);
 
-		method->ignore_ca_cert = nm_gconf_get_ignore_ca_cert (uuid, phase2);
+		/* Figure out if the user wants to ignore missing CA cert */
+		method->ignore_ca_cert = _get_ignore_ca_cert (uuid, method->phase2);
 	}
 
 	info = g_malloc0 (sizeof (NagDialogResponseInfo));
 	info->method = method;
 	info->connection = connection;
 
-	dialog = glade_xml_get_widget (method->nag_dialog_xml, "nag_user_dialog");
+	dialog = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "nag_user_dialog"));
 	g_assert (dialog);
 	g_signal_connect (dialog, "response", G_CALLBACK (nag_dialog_response_cb), info);
+	g_signal_connect (dialog, "delete-event", G_CALLBACK (nag_dialog_delete_event_cb), info);
+	g_object_weak_ref (G_OBJECT (dialog), nag_dialog_destroyed, info);
 
-	widget = glade_xml_get_widget (method->nag_dialog_xml, "content_label");
+	widget = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "content_label"));
 	g_assert (widget);
 
 	text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
@@ -208,11 +281,11 @@ eap_method_nag_init (EAPMethod *method,
 	gtk_label_set_markup (GTK_LABEL (widget), text);
 	g_free (text);
 
-	widget = glade_xml_get_widget (method->nag_dialog_xml, "ignore_button");
+	widget = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "ignore_button"));
 	gtk_button_set_label (GTK_BUTTON (widget), _("Ignore"));
 	g_assert (widget);
 
-	widget = glade_xml_get_widget (method->nag_dialog_xml, "change_button");
+	widget = GTK_WIDGET (gtk_builder_get_object (method->nag_builder, "change_button"));
 	gtk_button_set_label (GTK_BUTTON (widget), _("Choose CA Certificate"));
 	g_assert (widget);
 
@@ -220,14 +293,6 @@ eap_method_nag_init (EAPMethod *method,
 	return TRUE;
 }
 
-gboolean
-eap_method_get_ignore_ca_cert (EAPMethod *method)
-{
-	g_return_val_if_fail (method != NULL, FALSE);
-
-	return method->ignore_ca_cert;
-}
-
 void
 eap_method_phase2_update_secrets_helper (EAPMethod *method,
                                          NMConnection *connection,
@@ -242,7 +307,7 @@ eap_method_phase2_update_secrets_helper (EAPMethod *method,
 	g_return_if_fail (connection != NULL);
 	g_return_if_fail (combo_name != NULL);
 
-	combo = glade_xml_get_widget (method->xml, combo_name);
+	combo = GTK_WIDGET (gtk_builder_get_object (method->builder, combo_name));
 	g_assert (combo);
 
 	/* Let each EAP phase2 method try to update its secrets */
@@ -260,28 +325,56 @@ eap_method_phase2_update_secrets_helper (EAPMethod *method,
 	}
 }
 
-void
-eap_method_init (EAPMethod *method,
+EAPMethod *
+eap_method_init (gsize obj_size,
                  EMValidateFunc validate,
                  EMAddToSizeGroupFunc add_to_size_group,
                  EMFillConnectionFunc fill_connection,
                  EMUpdateSecretsFunc update_secrets,
                  EMDestroyFunc destroy,
-                 GladeXML *xml,
-                 GtkWidget *ui_widget,
-                 const char *default_field)
-{                 
-	method->refcount = 1;
+                 const char *ui_file,
+                 const char *ui_widget_name,
+                 const char *default_field,
+                 gboolean phase2)
+{
+	EAPMethod *method;
+	GError *error = NULL;
 
+	g_return_val_if_fail (obj_size > 0, NULL);
+	g_return_val_if_fail (ui_file != NULL, NULL);
+	g_return_val_if_fail (ui_widget_name != NULL, NULL);
+
+	method = g_slice_alloc0 (obj_size);
+	g_assert (method);
+
+	method->refcount = 1;
+	method->obj_size = obj_size;
 	method->validate = validate;
 	method->add_to_size_group = add_to_size_group;
 	method->fill_connection = fill_connection;
 	method->update_secrets = update_secrets;
 	method->destroy = destroy;
-
-	method->xml = xml;
-	method->ui_widget = ui_widget;
 	method->default_field = default_field;
+	method->phase2 = phase2;
+
+	method->builder = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (method->builder, ui_file, &error)) {
+		g_warning ("Couldn't load UI builder file %s: %s",
+		           ui_file, error->message);
+		eap_method_unref (method);
+		return NULL;
+	}
+
+	method->ui_widget = GTK_WIDGET (gtk_builder_get_object (method->builder, ui_widget_name));
+	if (!method->ui_widget) {
+		g_warning ("Couldn't load UI widget '%s' from UI file %s",
+		           ui_widget_name, ui_file);
+		eap_method_unref (method);
+		return NULL;
+	}
+	g_object_ref_sink (method->ui_widget);
+
+	return method;
 }
 
 
@@ -301,23 +394,27 @@ eap_method_unref (EAPMethod *method)
 	g_return_if_fail (method != NULL);
 	g_return_if_fail (method->refcount > 0);
 
-	g_assert (method->destroy);
-
 	method->refcount--;
 	if (method->refcount == 0) {
+		if (method->destroy)
+			method->destroy (method);
+
 		if (method->nag_dialog)
 			gtk_widget_destroy (method->nag_dialog);
-		if (method->nag_dialog_xml)
-			g_object_unref (method->nag_dialog_xml);
+		if (method->nag_builder)
+			g_object_unref (method->nag_builder);
 		g_free (method->ca_cert_chooser);
-		g_object_unref (method->xml);
-		g_object_unref (method->ui_widget);
-		(*(method->destroy)) (method);
+		if (method->builder)
+			g_object_unref (method->builder);
+		if (method->ui_widget)
+			g_object_unref (method->ui_widget);
+
+		g_slice_free1 (method->obj_size, method);
 	}
 }
 
 gboolean
-eap_method_validate_filepicker (GladeXML *xml,
+eap_method_validate_filepicker (GtkBuilder *builder,
                                 const char *name,
                                 guint32 item_type,
                                 const char *password,
@@ -334,7 +431,7 @@ eap_method_validate_filepicker (GladeXML *xml,
 		g_return_val_if_fail (strlen (password), FALSE);
 	}
 
-	widget = glade_xml_get_widget (xml, name);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, name));
 	g_assert (widget);
 	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
 	if (!filename)
@@ -394,6 +491,8 @@ find_tag (const char *tag, const char *buf, gsize len)
 
 static const char *pem_rsa_key_begin = "-----BEGIN RSA PRIVATE KEY-----";
 static const char *pem_dsa_key_begin = "-----BEGIN DSA PRIVATE KEY-----";
+static const char *pem_pkcs8_enc_key_begin = "-----BEGIN ENCRYPTED PRIVATE KEY-----";
+static const char *pem_pkcs8_dec_key_begin = "-----BEGIN PRIVATE KEY-----";
 static const char *pem_cert_begin = "-----BEGIN CERTIFICATE-----";
 static const char *proc_type_tag = "Proc-Type: 4,ENCRYPTED";
 static const char *dek_info_tag = "DEK-Info:";
@@ -423,6 +522,19 @@ file_has_extension (const char *filename, const char *extensions[])
 	return found;
 }
 
+static gboolean
+pem_file_is_encrypted (const char *buffer, gsize bytes_read)
+{
+	/* Check if the private key is encrypted or not by looking for the
+	 * old OpenSSL-style proc-type and dec-info tags.
+	 */
+	if (find_tag (proc_type_tag, (const char *) buffer, bytes_read)) {
+		if (find_tag (dek_info_tag, (const char *) buffer, bytes_read))
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static gboolean
 file_is_der_or_pem (const char *filename,
                     gboolean privkey,
@@ -431,9 +543,7 @@ file_is_der_or_pem (const char *filename,
 	int fd;
 	unsigned char buffer[8192];
 	ssize_t bytes_read;
-	guint16 der_tag = 0x8230;
 	gboolean success = FALSE;
-	gboolean encrypted = FALSE;
 
 	fd = open (filename, O_RDONLY);
 	if (fd < 0)
@@ -445,7 +555,7 @@ file_is_der_or_pem (const char *filename,
 	buffer[bytes_read] = '\0';
 
 	/* Check for DER signature */
-	if (!memcmp (buffer, &der_tag, 2)) {
+	if (bytes_read > 2 && buffer[0] == 0x30 && buffer[1] == 0x82) {
 		success = TRUE;
 		goto out;
 	}
@@ -454,23 +564,30 @@ file_is_der_or_pem (const char *filename,
 	if (privkey) {
 		if (find_tag (pem_rsa_key_begin, (const char *) buffer, bytes_read)) {
 			success = TRUE;
+			if (out_privkey_encrypted)
+				*out_privkey_encrypted = pem_file_is_encrypted ((const char *) buffer, bytes_read);
 			goto out;
 		}
 
 		if (find_tag (pem_dsa_key_begin, (const char *) buffer, bytes_read)) {
 			success = TRUE;
+			if (out_privkey_encrypted)
+				*out_privkey_encrypted = pem_file_is_encrypted ((const char *) buffer, bytes_read);
 			goto out;
 		}
 
-		/* Check if the private key is encrypted or not by looking for the
-		 * old OpenSSL-style proc-type and dec-info tags.
-		 */
-		if (out_privkey_encrypted) {
-			if (find_tag (proc_type_tag, (const char *) buffer, bytes_read)) {
-				if (find_tag (dek_info_tag, (const char *) buffer, bytes_read))
-					encrypted = TRUE;
-			}
-			*out_privkey_encrypted = encrypted;
+		if (find_tag (pem_pkcs8_enc_key_begin, (const char *) buffer, bytes_read)) {
+			success = TRUE;
+			if (out_privkey_encrypted)
+				*out_privkey_encrypted = TRUE;
+			goto out;
+		}
+
+		if (find_tag (pem_pkcs8_dec_key_begin, (const char *) buffer, bytes_read)) {
+			success = TRUE;
+			if (out_privkey_encrypted)
+				*out_privkey_encrypted = FALSE;
+			goto out;
 		}
 	} else {
 		if (find_tag (pem_cert_begin, (const char *) buffer, bytes_read)) {
